@@ -1,6 +1,6 @@
-/* xpm2wico - .xpm to Windows .ico format converter
+/* readicon.c - xpm2wico - .xpm to Windows .ico format converter
  *
- * Copyright (C) 2001 Wolfgang Sourdeau
+ * Copyright (C) 2001-2002 Wolfgang Sourdeau
  *
  * Author: Wolfgang Sourdeau <wolfgang@contre.com>
  *
@@ -19,61 +19,12 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <endian.h>
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "winutils.h"
 #include "xpm2wico.h"
 
-#define read_long(f) (Long) read_dword(f)
-
-Byte read_byte (FILE *file)
-{
-  Byte ret_byte;
-  
-  fread (&ret_byte, sizeof (Byte), 1, file);
-
-  return ret_byte;
-}
-
-Word read_word (FILE *file)
-{
-  Word ret_word;
-  
-#if (__BYTE_ORDER == __LITTLE_ENDIAN)
-  fread (&ret_word, 1, sizeof (Word), file);
-#else
-  int i = 0;
-
-  while (i < sizeof (Word)) {
-    fread (&ret_word, 1, sizeof (Byte), file);
-    ret_word >>= 8;
-    i++;
-  }
-#endif
-
-  return (ret_word);
-}
-
-DWord read_dword (FILE *file)
-{
-  DWord ret_dword;
-  
-#if (__BYTE_ORDER == __LITTLE_ENDIAN)
-  fread (&ret_dword, 1, sizeof (DWord), file);
-#else
-  int i = 0;
-
-  while (i < sizeof (DWord)) {
-    fread (&ret_dword, 1, sizeof (Byte), file);
-    ret_dword >>= 8;
-    i++;
-  }
-#endif
-
-  return ret_dword;
-}
-
 void print_icon_dir (IconDir *icon_dir)
 {
   printf ("reserved = %x\n"
@@ -83,7 +34,7 @@ void print_icon_dir (IconDir *icon_dir)
 	  icon_dir->idReserved,
 	  icon_dir->idType,
 	  icon_dir->idCount,
-	  icon_dir->idEntries);
+	  (int) icon_dir->idEntries);
 }
 
 void print_icon_dir_entry (IconDirEntry *icon_dir_entry)
@@ -101,8 +52,8 @@ void print_icon_dir_entry (IconDirEntry *icon_dir_entry)
 	  icon_dir_entry->bColorCount,
 	  icon_dir_entry->wPlanes,
 	  icon_dir_entry->wBitCount,
-	  icon_dir_entry->dwBytesInRes,
-	  icon_dir_entry->dwImageOffset);
+	  (int) icon_dir_entry->dwBytesInRes,
+	  (int) icon_dir_entry->dwImageOffset);
 }
 
 IconDirEntry *read_icon_dir_entry (FILE *file)
@@ -181,7 +132,6 @@ void read_icon_data (FILE *icon, IconDir *header)
 {
   IconDirEntry *icon_dir_entry;
   BitmapInfoHeader *icHeader;
-  RGBQuad *icColors;
 
   icon_dir_entry = header->idEntries;
 
@@ -224,6 +174,12 @@ int main (int argc, char *argv[])
   int ret_code;
   FILE *icon;
 
+  if (!argv[1])
+    {
+      fprintf (stderr, "Please specify an icon file as argument...\n");
+      exit (EXIT_FAILURE);
+    }
+	  
   icon = fopen (argv[1], "r");
 
   if (icon)
