@@ -1,6 +1,6 @@
-/* xpm2wico - .xpm to Windows .ico format converter
+/* xpm2wico.c - xpm2wico - .xpm to Windows .ico format converter
  *
- * Copyright (C) 2001 Wolfgang Sourdeau
+ * Copyright (C) 2001-2002 Wolfgang Sourdeau
  *
  * Author: Wolfgang Sourdeau <wolfgang@contre.com>
  *
@@ -23,7 +23,6 @@
 #include "../config.h"
 #endif
 
-#include <endian.h>
 #include <getopt.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -31,6 +30,8 @@
 #include <unistd.h>
 #include <X11/xpm.h>
 
+#include "winutils.h"
+#include "rgbtxt.h"
 #include "xpm2wico.h"
 
 #define display_warning(msg) \
@@ -69,66 +70,18 @@ struct _IconData
   DWord and_size;
 };
 
-DWord xpm_col_tbl_size;
-char *xpm_col_codes;
+extern char *rgb_txt_file;
 
-static int verbose = 0, force = 0;
+static DWord xpm_col_tbl_size;
+static char *xpm_col_codes;
 
-void
-write_byte (FILE *file, Byte byte)
-{
-  fwrite (&byte, sizeof (Byte), 1, file);
-}
 
-void
-write_word (FILE *file, Word word)
-{
-#if (__BYTE_ORDER == __LITTLE_ENDIAN)
-   fwrite (&word, sizeof (Word), 1, file);
-#else
-  int i = 0;
+int verbose = 0;
+static int force = 0;
 
-  while (i < sizeof (Word)) {
-    fputc (word & 0xff, file);
-    word >>= 8;
-    i++;
-  }
-#endif
-}
+extern RgbDictEntry *seek_dict_entry (char *name);
 
-void
-write_dword (FILE *file, DWord dword)
-{
-#if (__BYTE_ORDER == __LITTLE_ENDIAN)
-  fwrite (&dword, sizeof (DWord), 1, file);
-#else
-  int i = 0;
-
-  while (i < sizeof (DWord)) {
-    fputc (dword & 0xff, file);
-    dword >>= 8;
-    i++;
-  }
-#endif
-}
-
-void
-write_long (FILE *file, Long long_val)
-{
-#if (__BYTE_ORDER == __LITTLE_ENDIAN)
-  fwrite (&long_val, sizeof (Long), 1, file);
-#else
-  int i = 0;
-
-  while (i < sizeof (Long)) {
-    fputc (long_val & 0xff, file);
-    long_val >>= 8;
-    i++;
-  }
-#endif
-}
-
-int
+static int
 color_size (char *cstring)
 {
   int len, size;
@@ -140,7 +93,8 @@ color_size (char *cstring)
   return size;
 }
 
-int hex_value (char hexchar)
+static int
+hex_value (char hexchar)
 {
   int value;
 
@@ -157,7 +111,8 @@ int hex_value (char hexchar)
   return value;
 }
 
-Word bytes_per_pixel (Word ncolors)
+static Word
+bytes_per_pixel (Word ncolors)
 {
   Word bypp;
 
@@ -169,7 +124,7 @@ Word bytes_per_pixel (Word ncolors)
   return bypp;
 }
 
-DWord
+static DWord
 rounded_and (int nb)
 {
   DWord r_and;
@@ -179,17 +134,17 @@ rounded_and (int nb)
   return r_and;
 }
 
-Word
+static Word
 color_len (char *color)
 {
   Word len;
 
-  len = strlen (color + 1) / 3;
+  len = strlen (color) / 3;
 
   return len;
 }
 
-Byte
+static Byte
 norm_color (Word raw_color, Word len)
 {
   int count, factor;
@@ -212,7 +167,7 @@ norm_color (Word raw_color, Word len)
   return color;
 }
 
-Byte
+static Byte
 interp_color (char *color_str, Word len)
 {
   Byte color;
@@ -236,13 +191,13 @@ interp_color (char *color_str, Word len)
 }
 
 void
-color_convert (XpmColor *xpm_color, RGBQuad *wincolor)
+convert_rgb_string (char *rgb_string, RGBQuad *wincolor)
 {
   Word clen;
   char *char_ptr;
 
-  char_ptr = xpm_color->c_color + 1;
-  clen = color_len (xpm_color->c_color);
+  char_ptr = rgb_string + 1;
+  clen = color_len (rgb_string);
 
   wincolor->rgbRed = interp_color (char_ptr, clen);
   char_ptr += clen;
@@ -252,7 +207,39 @@ color_convert (XpmColor *xpm_color, RGBQuad *wincolor)
   wincolor->rgbReserved = 0;
 }
 
-void
+static void
+convert_rgb_rec (RgbDictEntry *rgb_record, RGBQuad *wincolor)
+{
+  wincolor->rgbRed = rgb_record->red;
+  wincolor->rgbGreen = rgb_record->green;
+  wincolor->rgbBlue = rgb_record->blue;
+  wincolor->rgbReserved = 0;
+}
+
+static void
+color_convert (XpmColor *xpm_color, RGBQuad *wincolor)
+{
+  RgbDictEntry *rgb_record;
+
+  if (*(xpm_color->c_color) == '#')
+    convert_rgb_string (xpm_color->c_color, wincolor);
+  else
+    {
+      rgb_record = seek_dict_entry (xpm_color->c_color);
+
+      if (!rgb_record)
+	{
+	  fprintf (stderr,
+		   "Error: `%s' is not a recognized color name\n",
+		   xpm_color->c_color);
+	  exit (EXIT_FAILURE);
+	}
+
+      convert_rgb_rec (rgb_record, wincolor);
+    }
+}
+
+static void
 gray_convert (XpmColor *xpm_color, RGBQuad *wincolor)
 {
   Word clen;
@@ -266,26 +253,26 @@ gray_convert (XpmColor *xpm_color, RGBQuad *wincolor)
   wincolor->rgbReserved = 0;
 }
 
-void
+static void
 gray4_convert (XpmColor *xpm_color, RGBQuad *wincolor)
 {
   display_error ("No 4-level grayscale color handling at this time.");
 }
 
-void
+static void
 mono_convert (XpmColor *xpm_color, RGBQuad *wincolor)
 {
   display_error ("No monochrome color handling at this time.");
 }
 
-void
+static void
 sym_convert (XpmColor *xpm_color, RGBQuad *wincolor)
 {
   display_error ("No symbolic color handling at this time.");
 }
 
 /* Not a real converter */
-void
+static void
 transp_convert (XpmColor *xpm_color, RGBQuad *wincolor)
 {
   wincolor->rgbRed = 0;
@@ -298,7 +285,7 @@ transp_convert (XpmColor *xpm_color, RGBQuad *wincolor)
    any xpm spec underhand to tell me how those different color schemes
    are supposed to work. Everything is based on X11/xpm.h. */
 
-ColorConverter
+static ColorConverter
 choose_converter (XpmColor *xpm_color)
 {
   ColorConverter cconverter;
@@ -352,7 +339,6 @@ RGBQuad
 convert_color (XpmColor *xpm_color)
 {
   RGBQuad win_color;
-  char *cstring;
   ColorConverter cconverter;
 
   /* This has to be redone each time convert_color is invoked. It is
@@ -374,7 +360,7 @@ convert_color (XpmColor *xpm_color)
   return win_color;
 }
 
-int
+static int
 is_valid_xpm_image (XpmImage *icon)
 {
   int ret_code;
@@ -411,7 +397,7 @@ is_valid_xpm_image (XpmImage *icon)
   return ret_code;
 }
 
-IconDir *
+static IconDir *
 new_icon_header ()
 {
   IconDir *header;
@@ -439,7 +425,7 @@ new_icon_dir_entry ()
   return icon_dir_entry;
 }
 
-IconDirEntry *
+static IconDirEntry *
 create_dir_entry (XpmImage *icon,
 		  IconData *icon_data,
 		  DWord ncolors)
@@ -462,7 +448,7 @@ create_dir_entry (XpmImage *icon,
   return icon_dir_entry;
 }
 
-BitmapInfoHeader *
+static BitmapInfoHeader *
 create_bm_ih (XpmImage *icon, Word bpp,
 	      DWord image_size, DWord nbr_colors)
 {
@@ -503,7 +489,7 @@ real_nbr_colors (DWord nbr_colors)
   return nbr;
 }
 
-int
+static int
 white_present (ColorMap *color_map)
 {
   RGBQuad *color_quad;
@@ -541,7 +527,7 @@ add_white (ColorMap *color_map, unsigned int clr_count)
   white->rgbReserved = 0;
 }
 
-ColorMap *
+static ColorMap *
 convert_color_map (XpmImage *icon, DWord ncolors)
 {
   unsigned int clr_count;
@@ -552,8 +538,9 @@ convert_color_map (XpmImage *icon, DWord ncolors)
       color_map = malloc (sizeof (ColorMap));
       color_map->nbr_colors = ncolors;
       color_map->color_list = malloc (sizeof (RGBQuad) * ncolors);
+      memset (color_map->color_list, 0, sizeof (RGBQuad) * ncolors);
 
-      for (clr_count = 0 ; clr_count < icon->ncolors; clr_count++)
+      for (clr_count = 0; clr_count < icon->ncolors; clr_count++)
 	*(color_map->color_list + clr_count) =
 	  convert_color (icon->colorTable + clr_count);
 
@@ -585,7 +572,7 @@ convert_color_map (XpmImage *icon, DWord ncolors)
   return color_map;
 }
 
-void
+static void
 put_24bit_color (DrawData *draw_data,
 		 Word x, Word y,
 		 RGBQuad *color)
@@ -617,7 +604,7 @@ put_8bit_color (DrawData *draw_data,
   *ptr = val;
 }
 
-Byte *
+static Byte *
 make_xor_mask (XpmImage *icon)
 {
   DWord mask_size, x, y;
@@ -658,7 +645,7 @@ make_xor_mask (XpmImage *icon)
   return mask;
 }
 
-Byte *
+static Byte *
 make_and_mask (XpmImage *icon, ColorMap *color_map)
 {
   DWord width, line_size, mask_size, x;
@@ -690,7 +677,7 @@ make_and_mask (XpmImage *icon, ColorMap *color_map)
   return mask;
 }
 
-void
+static void
 save_header (FILE *ico_file, IconDir *header)
 {
   write_word (ico_file, header->idReserved);
@@ -698,7 +685,7 @@ save_header (FILE *ico_file, IconDir *header)
   write_word (ico_file, header->idCount);
 }
 
-void
+static void
 save_dir_entry (FILE *ico_file, IconDirEntry *icon_dir_entry)
 {
   write_byte (ico_file, icon_dir_entry->bWidth);
@@ -711,7 +698,7 @@ save_dir_entry (FILE *ico_file, IconDirEntry *icon_dir_entry)
   write_dword (ico_file, icon_dir_entry->dwImageOffset);
 }
 
-void
+static void
 save_bm_ih (FILE *ico_file, BitmapInfoHeader *bm_ih)
 {
   write_dword (ico_file, bm_ih->biSize);
@@ -727,7 +714,8 @@ save_bm_ih (FILE *ico_file, BitmapInfoHeader *bm_ih)
   write_dword (ico_file, bm_ih->biClrImportant);
 }
 
-void save_RGBQuad (FILE *file, RGBQuad *rgb_quad)
+static void
+save_RGBQuad (FILE *file, RGBQuad *rgb_quad)
 {
   write_byte (file, rgb_quad->rgbBlue);
   write_byte (file, rgb_quad->rgbGreen);
@@ -735,7 +723,7 @@ void save_RGBQuad (FILE *file, RGBQuad *rgb_quad)
   write_byte (file, 0);
 }
 
-void
+static void
 save_color_map (FILE *ico_file, ColorMap *color_map)
 {
   DWord count;
@@ -743,6 +731,8 @@ save_color_map (FILE *ico_file, ColorMap *color_map)
 
   cur_quad = color_map->color_list;
 
+  if (verbose)
+    printf ("Colormap has %d colours.\n", color_map->nbr_colors);
   for (count = 0; count < color_map->nbr_colors; count++)
     {
       save_RGBQuad (ico_file, cur_quad);
@@ -750,22 +740,14 @@ save_color_map (FILE *ico_file, ColorMap *color_map)
     }
 }
 
-/*  void */
-/*  show_fpos (FILE *file) */
-/*  { */
-/*    fpos_t pos; */
-
-/*    fgetpos (file, &pos); */
-/*    printf ("file_pos = %d\n", pos); */
-/*  } */
-
-inline void
+static inline void
 save_mask (FILE *ico_file, DWord size, Byte *mask)
 {
   fwrite (mask, sizeof (Byte), size, ico_file);
 }
 
-int save_icon (char *file_name, IconData *icon_data)
+static int
+save_icon (char *file_name, IconData *icon_data)
 {
   int ret_code;
   FILE *ico_file;
@@ -802,9 +784,8 @@ int save_icon (char *file_name, IconData *icon_data)
   return ret_code;
 }
 
-int convert_icon (char *ico_file_name,
-		  XpmInfo *icon_info,
-		  XpmImage *icon)
+static int
+convert_icon (char *ico_file_name, XpmInfo *icon_info, XpmImage *icon)
 {
   int ret_code;
   IconData *icon_data;
@@ -846,7 +827,7 @@ int convert_icon (char *ico_file_name,
 
 /* As I am not sure every xpm file use the same symbol table, I feel
    better creating one dynamically... */
-void
+static void
 make_xpm_col_table (XpmImage *icon)
 {
   char *cur_color;
@@ -861,7 +842,7 @@ make_xpm_col_table (XpmImage *icon)
       cur_color = (icon->colorTable + ccount)->string;
 
       if (*cur_color == *(icon->colorTable->string))
-	found = !found;
+	found = 1;
       else
 	ccount++;
     }
@@ -880,7 +861,8 @@ make_xpm_col_table (XpmImage *icon)
   *(xpm_col_codes + max_colors) = 0;
 }
 
-int test_exist (char *file_name)
+static int
+test_exist (char *file_name)
 {
   int ret_code;
   char answer[2];
@@ -915,7 +897,7 @@ int test_exist (char *file_name)
   return ret_code;
 }
 
-void
+static void
 display_icon_info (XpmImage *icon)
 {
   printf ("Informations on input icon:\n"
@@ -926,7 +908,7 @@ display_icon_info (XpmImage *icon)
 	  icon->ncolors);
 }
 
-int
+static int
 process_icon (char *xpm_file_name, char *ico_file_name)
 {
   int x_ret_code, ret_code;
@@ -951,7 +933,8 @@ process_icon (char *xpm_file_name, char *ico_file_name)
 	}
       else
 	{
-	  perror (xpm_file_name);
+	  fprintf (stderr, "Problem reading \"%s\". This file is likely"
+		   " not an Xpm file.\n", xpm_file_name);
 	  ret_code = EXIT_FAILURE;
 	}
 
@@ -964,7 +947,7 @@ process_icon (char *xpm_file_name, char *ico_file_name)
   return ret_code;
 }
 
-char *
+static char *
 deduce_ico_name (char *file_name)
 {
   char *ret_name, *char_ptr, *last_slash_ptr;
@@ -1004,29 +987,25 @@ deduce_ico_name (char *file_name)
   return ret_name;
 }
 
-void
+static inline void
 copyright ()
 {
-  printf ("Copyright (C) 2001 Wolfgang Sourdeau\n",
-	  PACKAGE, VERSION);
+  printf ("Copyright (C) 2001, 2002 Wolfgang Sourdeau\n");
 }
 
-void
-basic_usage ()
+static inline void
+usage ()
 {
-  printf ("Usage: %s XPM_IN_FILE [ICO_OUT_FILE]\n",
-	  PACKAGE);
-  printf ("Try `%s --help' for more information.\n",
-	  PACKAGE);
+  printf ("Usage: %s [OPTIONS] XPM_IN_FILE [ICO_OUT_FILE]\n", PACKAGE);
 }
 
-void
+static inline void
 version ()
 {
   printf ("%s %s\n", PACKAGE, VERSION);
 }
 
-void
+static void
 help ()
 {
   printf ("This utility converts an xpm icon to a Windows ico file.\n"
@@ -1036,6 +1015,9 @@ help ()
 	  " message\n"
 	  "  -f, --force			overwrite output"
 	  " without asking\n"
+	  "  -r, --rgb-txt			specify a different"
+	  " rgb.txt from the\n"
+	  "				default \"" DEFAULT_RGB_TXT "\"\n"
 	  "  -h, --usage			display a short usage"
 	  " information\n"
 	  "  -v, --verbose			display informations"
@@ -1052,7 +1034,8 @@ help ()
 	  "Please report bugs to wolfgang@contre.com.\n");
 }
 
-int parse_args (int argc, char *argv[])
+static int
+parse_args (int argc, char *argv[])
 {
   int ret_code, gorc, stop_parsing, option_index;
   char *outfile;
@@ -1060,6 +1043,7 @@ int parse_args (int argc, char *argv[])
   struct option options[] =
     {
       { "force", 0, NULL, 'f' },
+      { "rgb-txt", 1, NULL, 'r' },
       { "help", 0, NULL, 'H' },
       { "usage", 0, NULL, 'h' },
       { "version", 0, NULL, 'V' },
@@ -1072,19 +1056,32 @@ int parse_args (int argc, char *argv[])
 
   while (!stop_parsing)
     {
-      gorc = getopt_long (argc, argv, "fHhVv", options, &option_index);
+      gorc = getopt_long (argc, argv, "fHhVvr:", options, &option_index);
 
       switch (gorc)
 	{
 	case -1:
-	  if (argv[optind + 1])
-	    outfile = argv[optind + 1];
+	  if (argv[optind])
+	    {
+	      if (argv[optind + 1])
+		outfile = argv[optind + 1];
+	      else
+		outfile = deduce_ico_name (argv[optind]);
+	      process_icon (argv[optind], outfile);
+	    }
 	  else
-	    outfile = deduce_ico_name (argv[optind]);
-
-	  process_icon (argv[optind], outfile);
+	    {
+	      fprintf (stderr,
+		       "No filename specified.\nUse \"--usage\" or \"--help\""
+		       " as command-line parameters to obtain usage"
+		       " information or generic help, respectively.\n");
+	      ret_code = EXIT_FAILURE;
+	    }
 	  stop_parsing = 1;
 	  break;
+	case 'r':
+	  rgb_txt_file = optarg;
+	  break;	  
 	case 'f':
 	  force = 1;
 	  break;
@@ -1093,7 +1090,7 @@ int parse_args (int argc, char *argv[])
 	  stop_parsing = 1;
 	  break;
 	case 'h':
-	  basic_usage ();
+	  usage ();
 	  stop_parsing = 1;
 	  break;
 	case 'V':
@@ -1104,8 +1101,8 @@ int parse_args (int argc, char *argv[])
 	  verbose = 1;
 	  break;
 	case '?':
-	  basic_usage ();
-	  printf ("Option `%s' not recognized.\n", argv[optind - 1]);
+	  usage ();
+	  printf ("Option \"%s\" not recognized.\n", argv[optind - 1]);
 	  stop_parsing = 1;
 	  ret_code = EXIT_FAILURE;
 	  break;
@@ -1119,10 +1116,50 @@ int parse_args (int argc, char *argv[])
   return ret_code;
 }
 
-int main (int argc, char *argv[])
+#ifdef __WIN32__
+static char*
+win32_local_dir ()
+{
+  static int inited = 0;
+  static char file_name[MAX_PATH];
+  char *sep;
+
+  if (!inited) 
+    {
+      GetModuleFileName (NULL, file_name, sizeof (file_name));
+      sep = strrchr (file_name, '\\');
+      *sep = '\0';
+      inited = 1;
+    }
+
+  return file_name;
+}
+
+static char*
+win32_local_file (char *file)
+{
+  char *cur_dir, *local_file;
+  int len;
+
+  cur_dir = win32_local_dir ();
+  len = strlen (file) + strlen (cur_dir) + 1;
+  local_file = malloc (len);
+  sprintf (local_file, "%s\\%s", cur_dir, file);
+
+  return local_file;
+}
+#endif /* __WIN32__ */
+
+int
+main (int argc, char *argv[])
 {
   int ret_code;
-  char *xpm_file_name, *ico_file_name;
+
+#ifdef __WIN32__
+  rgb_txt_file = win32_local_file (DEFAULT_RGB_TXT);
+#else /* __WIN32__ */
+  rgb_txt_file = DEFAULT_RGB_TXT;
+#endif /* __WIN32__ */
 
   ret_code = parse_args (argc, argv);
 
